@@ -1,7 +1,8 @@
-__version__ = '0.5.3'
+__version__ = '0.5.12'
+
 
 # amalgamate exclude jupyter_kernel parser_table parser_test_table pyghooks
-# amalgamate exclude winutils wizard pytest_plugin fs
+# amalgamate exclude winutils wizard pytest_plugin fs macutils
 import os as _os
 if _os.getenv('XONSH_DEBUG', ''):
     pass
@@ -17,18 +18,16 @@ else:
         _sys.modules['xonsh.lazyasd'] = __amalgam__
         lazyjson = __amalgam__
         _sys.modules['xonsh.lazyjson'] = __amalgam__
-        ansi_colors = __amalgam__
-        _sys.modules['xonsh.ansi_colors'] = __amalgam__
-        codecache = __amalgam__
-        _sys.modules['xonsh.codecache'] = __amalgam__
+        color_tools = __amalgam__
+        _sys.modules['xonsh.color_tools'] = __amalgam__
         platform = __amalgam__
         _sys.modules['xonsh.platform'] = __amalgam__
         pretty = __amalgam__
         _sys.modules['xonsh.pretty'] = __amalgam__
-        timings = __amalgam__
-        _sys.modules['xonsh.timings'] = __amalgam__
-        jobs = __amalgam__
-        _sys.modules['xonsh.jobs'] = __amalgam__
+        ansi_colors = __amalgam__
+        _sys.modules['xonsh.ansi_colors'] = __amalgam__
+        codecache = __amalgam__
+        _sys.modules['xonsh.codecache'] = __amalgam__
         lazyimps = __amalgam__
         _sys.modules['xonsh.lazyimps'] = __amalgam__
         parser = __amalgam__
@@ -49,24 +48,36 @@ else:
         _sys.modules['xonsh.events'] = __amalgam__
         foreign_shells = __amalgam__
         _sys.modules['xonsh.foreign_shells'] = __amalgam__
+        jobs = __amalgam__
+        _sys.modules['xonsh.jobs'] = __amalgam__
+        jsonutils = __amalgam__
+        _sys.modules['xonsh.jsonutils'] = __amalgam__
         lexer = __amalgam__
         _sys.modules['xonsh.lexer'] = __amalgam__
         openpy = __amalgam__
         _sys.modules['xonsh.openpy'] = __amalgam__
-        proc = __amalgam__
-        _sys.modules['xonsh.proc'] = __amalgam__
         xontribs = __amalgam__
         _sys.modules['xonsh.xontribs'] = __amalgam__
         dirstack = __amalgam__
         _sys.modules['xonsh.dirstack'] = __amalgam__
         inspectors = __amalgam__
         _sys.modules['xonsh.inspectors'] = __amalgam__
+        proc = __amalgam__
+        _sys.modules['xonsh.proc'] = __amalgam__
+        shell = __amalgam__
+        _sys.modules['xonsh.shell'] = __amalgam__
+        timings = __amalgam__
+        _sys.modules['xonsh.timings'] = __amalgam__
         xonfig = __amalgam__
         _sys.modules['xonsh.xonfig'] = __amalgam__
+        base_shell = __amalgam__
+        _sys.modules['xonsh.base_shell'] = __amalgam__
         environ = __amalgam__
         _sys.modules['xonsh.environ'] = __amalgam__
         tracer = __amalgam__
         _sys.modules['xonsh.tracer'] = __amalgam__
+        readline_shell = __amalgam__
+        _sys.modules['xonsh.readline_shell'] = __amalgam__
         replay = __amalgam__
         _sys.modules['xonsh.replay'] = __amalgam__
         aliases = __amalgam__
@@ -77,14 +88,8 @@ else:
         _sys.modules['xonsh.execer'] = __amalgam__
         imphooks = __amalgam__
         _sys.modules['xonsh.imphooks'] = __amalgam__
-        shell = __amalgam__
-        _sys.modules['xonsh.shell'] = __amalgam__
-        base_shell = __amalgam__
-        _sys.modules['xonsh.base_shell'] = __amalgam__
         main = __amalgam__
         _sys.modules['xonsh.main'] = __amalgam__
-        readline_shell = __amalgam__
-        _sys.modules['xonsh.readline_shell'] = __amalgam__
         del __amalgam__
     except ImportError:
         pass
