@@ -15,10 +15,10 @@ from xonsh.foreign_shells import foreign_shell_data
 from xonsh.jobs import jobs, fg, bg, clean_jobs
 from xonsh.platform import (ON_ANACONDA, ON_DARWIN, ON_WINDOWS, ON_FREEBSD,
                             ON_NETBSD)
-from xonsh.proc import foreground
+from xonsh.tools import unthreadable
 from xonsh.replay import replay_main
 from xonsh.timings import timeit_alias
-from xonsh.tools import argvquote, escape_windows_cmd_string, to_bool
+from xonsh.tools import argvquote, escape_windows_cmd_string, to_bool, swap_values
 from xonsh.xontribs import xontribs_main
 
 import xonsh.completers._aliases as xca
@@ -199,10 +199,18 @@ def _SOURCE_FOREIGN_PARSER():
     parser.add_argument('--seterrpostcmd', default=None, dest='seterrpostcmd',
                         help='command(s) to set exit-on-error after all'
                              'other commands.')
+    parser.add_argument('--overwrite-aliases', default=False, action='store_true',
+                        dest='overwrite_aliases',
+                        help='flag for whether or not sourced aliases should '
+                             'replace the current xonsh aliases.')
+    parser.add_argument('--show', default=False, action='store_true', dest='show',
+                        help='Will show the script output.')
+    parser.add_argument('-d', '--dry-run', default=False, action='store_true',
+                        dest='dryrun', help='Will not actually source the file.')
     return parser
 
 
-def source_foreign(args, stdin=None):
+def source_foreign(args, stdin=None, stdout=None, stderr=None):
     """Sources a file written in a foreign shell language."""
     ns = _SOURCE_FOREIGN_PARSER.parse_args(args)
     if ns.prevcmd is not None:
@@ -224,10 +232,15 @@ def source_foreign(args, stdin=None):
                                           sourcer=ns.sourcer,
                                           use_tmpfile=ns.use_tmpfile,
                                           seterrprevcmd=ns.seterrprevcmd,
-                                          seterrpostcmd=ns.seterrpostcmd)
+                                          seterrpostcmd=ns.seterrpostcmd,
+                                          show=ns.show,
+                                          dryrun=ns.dryrun)
     if fsenv is None:
-        return (None, 'xonsh: error: Source failed: '
-                      '{}\n'.format(ns.prevcmd), 1)
+        if ns.dryrun:
+            return
+        else:
+            msg = 'xonsh: error: Source failed: {}\n'.format(ns.prevcmd)
+            return (None, msg, 1)
     # apply results
     env = builtins.__xonsh_env__
     denv = env.detype()
@@ -244,7 +257,13 @@ def source_foreign(args, stdin=None):
     for k, v in fsaliases.items():
         if k in baliases and v == baliases[k]:
             continue  # no change from original
-        baliases[k] = v
+        elif ns.overwrite_aliases or k not in baliases:
+            baliases[k] = v
+        else:
+            msg = ('Skipping application of {0!r} alias from {1!r} '
+                   'since it shares a name with an existing xonsh alias. '
+                   'Use "--overwrite-alias" option to apply it anyway.')
+            print(msg.format(k, ns.shell), file=stderr)
 
 
 def source_alias(args, stdin=None):
@@ -270,8 +289,10 @@ def source_alias(args, stdin=None):
             src = fp.read()
         if not src.endswith('\n'):
             src += '\n'
-        with env.swap(ARGS=args[i+1:]):
-            builtins.execx(src, 'exec', builtins.__xonsh_ctx__, filename=fpath)
+        ctx = builtins.__xonsh_ctx__
+        updates = {'__file__': fpath, '__name__': os.path.abspath(fpath)}
+        with env.swap(ARGS=args[i+1:]), swap_values(ctx, updates):
+            builtins.execx(src, 'exec', ctx, filename=fpath)
 
 
 def source_cmd(args, stdin=None):
@@ -350,12 +371,13 @@ def xonfig(args, stdin=None):
     return xonfig_main(args)
 
 
-@foreground
-def trace(args, stdin=None):
+@unthreadable
+def trace(args, stdin=None, stdout=None, stderr=None, spec=None):
     """Runs the xonsh tracer utility."""
     from xonsh.tracer import tracermain  # lazy import
     try:
-        return tracermain(args)
+        return tracermain(args, stdin=stdin, stdout=stdout,
+                          stderr=stderr, spec=spec)
     except SystemExit:
         pass
 
@@ -380,6 +402,27 @@ def showcmd(args, stdin=None):
         sys.displayhook(args)
 
 
+def detect_xip_alias():
+    """
+    Determines the correct invokation to get xonsh's pip
+    """
+    if not getattr(sys, 'executable', None):
+        return lambda args, stdin=None: ("", "Sorry, unable to run pip on your system (missing sys.executable)", 1)
+
+    basecmd = [sys.executable, '-m', 'pip']
+    try:
+        if ON_WINDOWS:
+            # XXX: Does windows have an installation mode that requires UAC?
+            return basecmd
+        elif not os.access(os.path.dirname(sys.executable), os.W_OK):
+            return ['sudo'] + basecmd
+        else:
+            return basecmd
+    except Exception:
+        # Something freaky happened, return something that'll probably work
+        return basecmd
+
+
 def make_default_aliases():
     """Creates a new default aliases dictionary."""
     default_aliases = {
@@ -410,7 +453,8 @@ def make_default_aliases():
         'ipynb': ['jupyter', 'notebook', '--no-browser'],
         'which': xxw.which,
         'xontrib': xontribs_main,
-        'completer': xca.completer_alias
+        'completer': xca.completer_alias,
+        'xip': detect_xip_alias(),
     }
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
@@ -445,7 +489,7 @@ def make_default_aliases():
         if not locate_binary('sudo'):
             import xonsh.winutils as winutils
 
-            def sudo(args, sdin=None):
+            def sudo(args):
                 if len(args) < 1:
                     print('You need to provide an executable to run as '
                           'Administrator.')
