@@ -2,6 +2,7 @@
 """The xonsh abstract syntax tree node."""
 # These are imported into our module namespace for the benefit of parser.py.
 # pylint: disable=unused-import
+import sys
 from ast import (
     Module, Num, Expr, Str, Bytes, UnaryOp, UAdd, USub, Invert,
     BinOp, Add, Sub, Mult, Div, FloorDiv, Mod, Pow, Compare, Lt, Gt,
@@ -20,7 +21,7 @@ from ast import Ellipsis as EllipsisNode
 import textwrap
 import itertools
 
-from xonsh.tools import subproc_toks, find_next_break
+from xonsh.tools import subproc_toks, find_next_break, get_logical_line
 from xonsh.platform import PYTHON_VERSION_INFO
 
 if PYTHON_VERSION_INFO >= (3, 5, 0):
@@ -91,11 +92,25 @@ def min_col(node):
 def max_col(node):
     """Returns the maximum col_offset of the node and all sub-nodes."""
     col = getattr(node, 'max_col', None)
-    if col is None:
-        col = max(map(get_col, walk(node)))
+    if col is not None:
+        return col
+    highest = max(walk(node), key=get_col)
+    col = highest.col_offset + node_len(highest)
     return col
 
 
+def node_len(node):
+    """The length of a node as a string"""
+    val = 0
+    for n in walk(node):
+        if isinstance(n, Name):
+            val += len(n.id)
+        elif isinstance(n, Attribute):
+            val += 1 + (len(n.attr) if isinstance(n.attr, str) else 0)
+        # this may need to be added to for more nodes as more cases are found
+    return val
+
+
 def get_id(node, default=None):
     """Gets the id attribute of a node, or returns a default."""
     return getattr(node, 'id', default)
@@ -171,8 +186,10 @@ class CtxAwareTransformer(NodeTransformer):
         self.lines = None
         self.mode = None
         self._nwith = 0
+        self.filename = '<xonsh-code>'
+        self.debug_level = 0
 
-    def ctxvisit(self, node, inp, ctx, mode='exec'):
+    def ctxvisit(self, node, inp, ctx, mode='exec', filename=None, debug_level=0):
         """Transforms the node in a context-dependent way.
 
         Parameters
@@ -183,12 +200,18 @@ class CtxAwareTransformer(NodeTransformer):
             The input code in string format.
         ctx : dict
             The root context to use.
+        filename : str, optional
+            File we are to transform.
+        debug_level : int, optional
+            Debugging level to use in lexing and parsing.
 
         Returns
         -------
         node : ast.AST
             The transformed node.
         """
+        self.filename = self.filename if filename is None else filename
+        self.debug_level = debug_level
         self.lines = inp.splitlines()
         self.contexts = [ctx, set()]
         self.mode = mode
@@ -215,33 +238,48 @@ class CtxAwareTransformer(NodeTransformer):
 
     def try_subproc_toks(self, node, strip_expr=False):
         """Tries to parse the line of the node as a subprocess."""
-        line = self.lines[node.lineno - 1]
+        line, nlogical, idx = get_logical_line(self.lines, node.lineno - 1)
         if self.mode == 'eval':
             mincol = len(line) - len(line.lstrip())
             maxcol = None
         else:
-            mincol = min_col(node)
+            mincol = max(min_col(node) - 1, 0)
             maxcol = max_col(node)
             if mincol == maxcol:
                 maxcol = find_next_break(line, mincol=mincol,
                                          lexer=self.parser.lexer)
+            elif nlogical > 1:
+                maxcol = None
             else:
                 maxcol += 1
-        spline = subproc_toks(line,
-                              mincol=mincol,
-                              maxcol=maxcol,
-                              returnline=False,
-                              lexer=self.parser.lexer)
+        spline = subproc_toks(line, mincol=mincol, maxcol=maxcol,
+                              returnline=False, lexer=self.parser.lexer)
+        if spline is None or len(spline) < len(line[mincol:maxcol]) + 2:
+            # failed to get something consistent, try greedy wrap
+            # The +2 comes from "![]" being length 3, minus 1 since maxcol
+            # is one beyond the total length for slicing
+            spline = subproc_toks(line, mincol=mincol, maxcol=maxcol,
+                                  returnline=False, lexer=self.parser.lexer,
+                                  greedy=True)
         if spline is None:
             return node
         try:
-            newnode = self.parser.parse(spline, mode=self.mode)
+            newnode = self.parser.parse(spline, mode=self.mode,
+                                        filename=self.filename,
+                                        debug_level=(self.debug_level > 2))
             newnode = newnode.body
             if not isinstance(newnode, AST):
                 # take the first (and only) Expr
                 newnode = newnode[0]
             increment_lineno(newnode, n=node.lineno - 1)
             newnode.col_offset = node.col_offset
+            if self.debug_level > 1:
+                msg = ('{0}:{1}:{2}{3} - {4}\n'
+                       '{0}:{1}:{2}{3} + {5}')
+                mstr = '' if maxcol is None else ':' + str(maxcol)
+                msg = msg.format(self.filename, node.lineno,
+                                 mincol, mstr, line, spline)
+                print(msg, file=sys.stderr)
         except SyntaxError:
             newnode = node
         if strip_expr and isinstance(newnode, Expr):
