@@ -1,16 +1,15 @@
 # -*- coding: utf-8 -*-
 """Implements the xonsh executer."""
-import re
 import sys
 import types
 import inspect
 import builtins
-import warnings
-import collections.abc as abc
+import collections.abc as cabc
 
 from xonsh.ast import CtxAwareTransformer
 from xonsh.parser import Parser
-from xonsh.tools import subproc_toks, find_next_break
+from xonsh.tools import (subproc_toks, find_next_break, get_logical_line,
+                         replace_logical_line)
 from xonsh.built_ins import load_builtins, unload_builtins
 
 
@@ -18,7 +17,7 @@ class Execer(object):
     """Executes xonsh code in a context."""
 
     def __init__(self, filename='<xonsh-code>', debug_level=0, parser_args=None,
-                 unload=True, config=None, login=True, xonsh_ctx=None):
+                 unload=True, xonsh_ctx=None, scriptcache=True, cacheall=False):
         """Parameters
         ----------
         filename : str, optional
@@ -29,31 +28,39 @@ class Execer(object):
             Arguments to pass down to the parser.
         unload : bool, optional
             Whether or not to unload xonsh builtins upon deletion.
-        config : str, optional
-            Path to configuration file.
         xonsh_ctx : dict or None, optional
             Xonsh xontext to load as builtins.__xonsh_ctx__
+        scriptcache : bool, optional
+            Whether or not to use a precompiled bytecode cache when execing
+            code, default: True.
+        cacheall : bool, optional
+            Whether or not to cache all xonsh code, and not just files. If this
+            is set to true, it will cache command line input too, default: False.
         """
         parser_args = parser_args or {}
         self.parser = Parser(**parser_args)
         self.filename = filename
         self.debug_level = debug_level
         self.unload = unload
+        self.scriptcache = scriptcache
+        self.cacheall = cacheall
         self.ctxtransformer = CtxAwareTransformer(self.parser)
-        load_builtins(execer=self, config=config, login=login, ctx=xonsh_ctx)
+        load_builtins(execer=self, ctx=xonsh_ctx)
 
     def __del__(self):
         if self.unload:
             unload_builtins()
 
-    def parse(self, input, ctx, mode='exec', transform=True):
+    def parse(self, input, ctx, mode='exec', filename=None, transform=True):
         """Parses xonsh code in a context-aware fashion. For context-free
         parsing, please use the Parser class directly or pass in
         transform=False.
         """
+        if filename is None:
+            filename = self.filename
         if not transform:
-            return self.parser.parse(input, filename=self.filename, mode=mode,
-                                     debug_level=(self.debug_level > 1))
+            return self.parser.parse(input, filename=filename, mode=mode,
+                                     debug_level=(self.debug_level > 2))
 
         # Parsing actually happens in a couple of phases. The first is a
         # shortcut for a context-free parser. Normally, all subprocess
@@ -69,7 +76,7 @@ class Execer(object):
         # tokens for all of the Python rules. The lazy way implemented here
         # is to parse a line a second time with a $() wrapper if it fails
         # the first time. This is a context-free phase.
-        tree, input = self._parse_ctx_free(input, mode=mode)
+        tree, input = self._parse_ctx_free(input, mode=mode, filename=filename)
         if tree is None:
             return None
 
@@ -81,9 +88,10 @@ class Execer(object):
         # it also is valid as a subprocess line.
         if ctx is None:
             ctx = set()
-        elif isinstance(ctx, abc.Mapping):
+        elif isinstance(ctx, cabc.Mapping):
             ctx = set(ctx.keys())
-        tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode)
+        tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode,
+                                            debug_level=self.debug_level)
         return tree
 
     def compile(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
@@ -98,60 +106,64 @@ class Execer(object):
             glbs = frame.f_globals if glbs is None else glbs
             locs = frame.f_locals if locs is None else locs
         ctx = set(dir(builtins)) | set(glbs.keys()) | set(locs.keys())
-        tree = self.parse(input, ctx, mode=mode, transform=transform)
+        tree = self.parse(input, ctx, mode=mode, filename=filename,
+                          transform=transform)
         if tree is None:
             return None  # handles comment only input
-        if transform:
-            with warnings.catch_warnings():
-                # we do some funky things with blocks that cause warnings
-                warnings.simplefilter('ignore', SyntaxWarning)
-                code = compile(tree, filename, mode)
-        else:
-            code = compile(tree, filename, mode)
+        code = compile(tree, filename, mode)
         return code
 
     def eval(self, input, glbs=None, locs=None, stacklevel=2,
-                transform=True):
+             filename=None, transform=True):
         """Evaluates (and returns) xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
         else:
+            if filename is None:
+                filename = self.filename
             code = self.compile(input=input,
                                 glbs=glbs,
                                 locs=locs,
                                 mode='eval',
                                 stacklevel=stacklevel,
+                                filename=filename,
                                 transform=transform)
         if code is None:
             return None  # handles comment only input
         return eval(code, glbs, locs)
 
     def exec(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
-                transform=True):
+             filename=None, transform=True):
         """Execute xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
         else:
+            if filename is None:
+                filename = self.filename
             code = self.compile(input=input,
                                 glbs=glbs,
                                 locs=locs,
                                 mode=mode,
                                 stacklevel=stacklevel,
+                                filename=filename,
                                 transform=transform)
         if code is None:
             return None  # handles comment only input
         return exec(code, glbs, locs)
 
-    def _parse_ctx_free(self, input, mode='exec'):
+    def _parse_ctx_free(self, input, mode='exec', filename=None):
         last_error_line = last_error_col = -1
         parsed = False
         original_error = None
+        greedy = False
+        if filename is None:
+            filename = self.filename
         while not parsed:
             try:
                 tree = self.parser.parse(input,
-                                         filename=self.filename,
+                                         filename=filename,
                                          mode=mode,
-                                         debug_level=(self.debug_level > 1))
+                                         debug_level=(self.debug_level > 2))
                 parsed = True
             except IndentationError as e:
                 if original_error is None:
@@ -164,19 +176,19 @@ class Execer(object):
                 if (e.loc is None) or (last_error_line == e.loc.lineno and
                                        last_error_col in (e.loc.column + 1,
                                                           e.loc.column)):
-                    raise original_error
+                    raise original_error from None
                 last_error_col = e.loc.column
                 last_error_line = e.loc.lineno
                 idx = last_error_line - 1
                 lines = input.splitlines()
-                line = lines[idx]
+                line, nlogical, idx = get_logical_line(lines, idx)
                 if input.endswith('\n'):
                     lines.append('')
                 if len(line.strip()) == 0:
                     # whitespace only lines are not valid syntax in Python's
                     # interactive mode='single', who knew?! Just ignore them.
                     # this might cause actual sytax errors to have bad line
-                    # numbers reported, but should only effect interactive mode
+                    # numbers reported, but should only affect interactive mode
                     del lines[idx]
                     last_error_line = last_error_col = -1
                     input = '\n'.join(lines)
@@ -189,9 +201,16 @@ class Execer(object):
                     if prev_indent == curr_indent:
                         raise original_error
                 lexer = self.parser.lexer
-                maxcol = find_next_break(line, mincol=last_error_col,
-                                         lexer=lexer)
-                sbpline = subproc_toks(line, returnline=True,
+                maxcol = None if greedy else find_next_break(line,
+                                                             mincol=last_error_col,
+                                                             lexer=lexer)
+                if not greedy and maxcol in (e.loc.column + 1, e.loc.column):
+                    # go greedy the first time if the syntax error was because
+                    # we hit an end token out of place. This usually indicates
+                    # a subshell or maybe a macro.
+                    greedy = True
+                    maxcol = None
+                sbpline = subproc_toks(line, returnline=True, greedy=greedy,
                                        maxcol=maxcol, lexer=lexer)
                 if sbpline is None:
                     # subprocess line had no valid tokens,
@@ -201,24 +220,33 @@ class Execer(object):
                         last_error_line = last_error_col = -1
                         input = '\n'.join(lines)
                         continue
+                    elif not greedy:
+                        greedy = True
+                        continue
                     else:
                         # or for some other syntax error
                         raise original_error
                 elif sbpline[last_error_col:].startswith('![![') or \
-                     sbpline.lstrip().startswith('![!['):
+                        sbpline.lstrip().startswith('![!['):
                     # if we have already wrapped this in subproc tokens
                     # and it still doesn't work, adding more won't help
                     # anything
-                    raise original_error
+                    if not greedy:
+                        greedy = True
+                        continue
+                    else:
+                        raise original_error
                 else:
-                    if self.debug_level:
+                    # print some debugging info
+                    if self.debug_level > 1:
                         msg = ('{0}:{1}:{2}{3} - {4}\n'
                                '{0}:{1}:{2}{3} + {5}')
                         mstr = '' if maxcol is None else ':' + str(maxcol)
                         msg = msg.format(self.filename, last_error_line,
                                          last_error_col, mstr, line, sbpline)
                         print(msg, file=sys.stderr)
-                    lines[idx] = sbpline
+                    # replace the line
+                    replace_logical_line(lines, sbpline, idx, nlogical)
                 last_error_col += 3
                 input = '\n'.join(lines)
         return tree, input
