@@ -4,7 +4,7 @@ import io
 import json
 import weakref
 import contextlib
-import collections.abc as abc
+import collections.abc as cabc
 
 
 def _to_json_with_size(obj, offset=0, sort_keys=False):
@@ -12,7 +12,7 @@ def _to_json_with_size(obj, offset=0, sort_keys=False):
         s = json.dumps(obj)
         o = offset
         n = size = len(s.encode())  # size in bytes
-    elif isinstance(obj, abc.Mapping):
+    elif isinstance(obj, cabc.Mapping):
         s = '{'
         j = offset + 1
         o = {}
@@ -35,7 +35,7 @@ def _to_json_with_size(obj, offset=0, sort_keys=False):
         n = len(s)
         o['__total__'] = offset
         size['__total__'] = n
-    elif isinstance(obj, abc.Sequence):
+    elif isinstance(obj, cabc.Sequence):
         s = '['
         j = offset + 1
         o = []
@@ -94,7 +94,7 @@ def ljdump(obj, fp, sort_keys=False):
     fp.write(s)
 
 
-class LJNode(abc.Mapping, abc.Sequence):
+class LJNode(cabc.Mapping, cabc.Sequence):
     """A proxy node for JSON nodes. Acts as both sequence and mapping."""
 
     def __init__(self, offsets, sizes, root):
@@ -110,8 +110,8 @@ class LJNode(abc.Mapping, abc.Sequence):
         self.offsets = offsets
         self.sizes = sizes
         self.root = root
-        self.is_mapping = isinstance(self.offsets, abc.Mapping)
-        self.is_sequence = isinstance(self.offsets, abc.Sequence)
+        self.is_mapping = isinstance(self.offsets, cabc.Mapping)
+        self.is_sequence = isinstance(self.offsets, cabc.Sequence)
 
     def __len__(self):
         # recall that for maps, the '__total__' key is added and for
@@ -137,7 +137,7 @@ class LJNode(abc.Mapping, abc.Sequence):
                 f.seek(self.root.dloc + offset)
                 s = f.read(size)
             val = json.loads(s)
-        elif isinstance(offset, (abc.Mapping, abc.Sequence)):
+        elif isinstance(offset, (cabc.Mapping, cabc.Sequence)):
             val = LJNode(offset, size, self.root)
         else:
             raise TypeError('incorrect types for offset node')
@@ -204,8 +204,8 @@ class LazyJSON(LJNode):
             self._f = open(f, 'r', newline='\n')
         self._load_index()
         self.root = weakref.proxy(self)
-        self.is_mapping = isinstance(self.offsets, abc.Mapping)
-        self.is_sequence = isinstance(self.offsets, abc.Sequence)
+        self.is_mapping = isinstance(self.offsets, cabc.Mapping)
+        self.is_sequence = isinstance(self.offsets, cabc.Sequence)
 
     def __del__(self):
         self.close()
