@@ -4,14 +4,22 @@ on a platform.
 """
 import os
 import sys
+import ctypes
 import signal
 import pathlib
+import builtins
 import platform
 import functools
 import subprocess
+import collections
 import importlib.util
 
 from xonsh.lazyasd import LazyBool, lazyobject, lazybool
+# do not import any xonsh-modules here to avoid circular dependencies
+
+FD_STDIN = 0
+FD_STDOUT = 1
+FD_STDERR = 2
 
 
 @lazyobject
@@ -25,9 +33,6 @@ def distro():
     return d
 
 
-# do not import any xonsh-modules here to avoid circular dependencies
-
-
 #
 # OS
 #
@@ -50,9 +55,18 @@ ON_FREEBSD = LazyBool(lambda: (sys.platform.startswith('freebsd')),
 ON_NETBSD = LazyBool(lambda: (sys.platform.startswith('netbsd')),
                      globals(), 'ON_NETBSD')
 """``True`` if on a NetBSD operating system, else ``False``."""
-ON_BSD = LazyBool(lambda: ON_FREEBSD or ON_NETBSD,
-                  globals(), 'ON_BSD')
-"""``True`` if on a BSD operating system, else ``False``."""
+
+
+@lazybool
+def ON_BSD():
+    """``True`` if on a BSD operating system, else ``False``."""
+    return bool(ON_FREEBSD) or bool(ON_NETBSD)
+
+
+@lazybool
+def ON_BEOS():
+    """True if we are on BeOS or Haiku."""
+    return sys.platform == 'beos5' or sys.platform == 'haiku1'
 
 
 #
@@ -61,6 +75,14 @@ ON_BSD = LazyBool(lambda: ON_FREEBSD or ON_NETBSD,
 
 PYTHON_VERSION_INFO = sys.version_info[:3]
 """ Version of Python interpreter as three-value tuple. """
+
+
+@lazyobject
+def PYTHON_VERSION_INFO_BYTES():
+    """The python version info tuple in a canonical bytes form."""
+    return '.'.join(map(str, sys.version_info)).encode()
+
+
 ON_ANACONDA = LazyBool(
     lambda: any(s in sys.version for s in {'Anaconda', 'Continuum', 'conda-forge'}),
     globals(), 'ON_ANACONDA')
@@ -165,6 +187,30 @@ def pathbasename(p):
     """
     return pathsplit(p)[-1]
 
+
+@lazyobject
+def expanduser():
+    """Dispatches to the correct platform-dependent expanduser() function."""
+    if ON_WINDOWS:
+        return windows_expanduser
+    else:
+        return os.path.expanduser
+
+
+def windows_expanduser(path):
+    """A Windows-specific expanduser() function for xonsh. This is needed
+    since os.path.expanduser() does not check on Windows if the user actually
+    exists. This restircts expanding the '~' if it is not followed by a
+    separator. That is only '~/' and '~\' are expanded.
+    """
+    if not path.startswith('~'):
+        return path
+    elif len(path) < 2 or path[1] in seps:
+        return os.path.expanduser(path)
+    else:
+        return path
+
+
 # termios tc(get|set)attr indexes.
 IFLAG = 0
 OFLAG = 1
@@ -285,22 +331,99 @@ def windows_bash_command():
     # Check that bash is on path otherwise try the default directory
     # used by Git for windows
     wbc = 'bash'
-    try:
-        subprocess.check_call([wbc, '--version'],
-                              stdout=subprocess.PIPE,
-                              stderr=subprocess.PIPE)
-    except (FileNotFoundError, subprocess.CalledProcessError):
-        gfwp = git_for_windows_path()
-        if gfwp:
-            bashcmd = os.path.join(gfwp, 'bin\\bash.exe')
-            if os.path.isfile(bashcmd):
-                wbc = bashcmd
+    cmd_cache = builtins.__xonsh_commands_cache__
+    bash_on_path = cmd_cache.lazy_locate_binary('bash', ignore_alias=True)
+    if bash_on_path:
+        try:
+            out = subprocess.check_output([bash_on_path, '--version'],
+                                          stderr=subprocess.PIPE,
+                                          universal_newlines=True)
+        except subprocess.CalledProcessError:
+            bash_works = False
+        else:
+            # Check if Bash is from the "Windows Subsystem for Linux" (WSL)
+            # which can't be used by xonsh foreign-shell/completer
+            bash_works = out and 'pc-linux-gnu' not in out.splitlines()[0]
+
+        if bash_works:
+            wbc = bash_on_path
+        else:
+            gfwp = git_for_windows_path()
+            if gfwp:
+                bashcmd = os.path.join(gfwp, 'bin\\bash.exe')
+                if os.path.isfile(bashcmd):
+                    wbc = bashcmd
     return wbc
 
 #
 # Environment variables defaults
 #
 
+if ON_WINDOWS:
+    class OSEnvironCasePreserving(collections.MutableMapping):
+        """ Case-preseving wrapper for os.environ on Windows.
+            It uses nt.environ to get the correct cased keys on
+            initialization. It also preserves the case of any variables
+            add after initialization.
+        """
+        def __init__(self):
+            import nt
+            self._upperkeys = dict((k.upper(), k) for k in nt.environ)
+
+        def _sync(self):
+            """ Ensure that the case sensitive map of the keys are
+                in sync with os.environ
+            """
+            envkeys = set(os.environ.keys())
+            for key in envkeys.difference(self._upperkeys):
+                self._upperkeys[key] = key.upper()
+            for key in set(self._upperkeys).difference(envkeys):
+                del self._upperkeys[key]
+
+        def __contains__(self, k):
+            self._sync()
+            return k.upper() in self._upperkeys
+
+        def __len__(self):
+            self._sync()
+            return len(self._upperkeys)
+
+        def __iter__(self):
+            self._sync()
+            return iter(self._upperkeys.values())
+
+        def __getitem__(self, k):
+            self._sync()
+            return os.environ[k]
+
+        def __setitem__(self, k, v):
+            self._sync()
+            self._upperkeys[k.upper()] = k
+            os.environ[k] = v
+
+        def __delitem__(self, k):
+            self._sync()
+            if k.upper() in self._upperkeys:
+                del self._upperkeys[k.upper()]
+                del os.environ[k]
+
+        def getkey_actual_case(self, k):
+            self._sync()
+            return self._upperkeys.get(k.upper())
+
+
+@lazyobject
+def os_environ():
+    """This dispatches to the correct, case-senstive version of os.envrion.
+    This is mainly a problem for Windows. See #2024 for more details.
+    This can probably go away once support for Python v3.5 or v3.6 is
+    dropped.
+    """
+    if ON_WINDOWS:
+        return OSEnvironCasePreserving()
+    else:
+        return os.environ
+
 
 @functools.lru_cache(1)
 def bash_command():
@@ -354,3 +477,60 @@ def PATH_DEFAULT():
     else:
         pd = ()
     return pd
+
+
+#
+# libc
+#
+@lazyobject
+def LIBC():
+    """The platform dependent libc implementation."""
+    if ON_DARWIN:
+        libc = ctypes.CDLL(ctypes.util.find_library("c"))
+    elif ON_CYGWIN:
+        libc = ctypes.CDLL('cygwin1.dll')
+    elif ON_BSD:
+        try:
+            libc = ctypes.CDLL('libc.so')
+        except AttributeError:
+            libc = None
+        except OSError:
+            # OS X; can't use ctypes.util.find_library because that creates
+            # a new process on Linux, which is undesirable.
+            try:
+                libc = ctypes.CDLL('libc.dylib')
+            except OSError:
+                libc = None
+    elif ON_POSIX:
+        try:
+            libc = ctypes.CDLL('libc.so')
+        except AttributeError:
+            libc = None
+        except OSError:
+            # Debian and derivatives do the wrong thing because /usr/lib/libc.so
+            # is a GNU ld script rather than an ELF object. To get around this, we
+            # have to be more specific.
+            # We don't want to use ctypes.util.find_library because that creates a
+            # new process on Linux. We also don't want to try too hard because at
+            # this point we're already pretty sure this isn't Linux.
+            try:
+                libc = ctypes.CDLL('libc.so.6')
+            except OSError:
+                libc = None
+        if not hasattr(libc, 'sysinfo'):
+            # Not Linux.
+            libc = None
+    elif ON_WINDOWS:
+        if hasattr(ctypes, 'windll') and hasattr(ctypes.windll, 'kernel32'):
+            libc = ctypes.windll.kernel32
+        else:
+            try:
+                # Windows CE uses the cdecl calling convention.
+                libc = ctypes.CDLL('coredll.lib')
+            except (AttributeError, OSError):
+                libc = None
+    elif ON_BEOS:
+        libc = ctypes.CDLL('libroot.so')
+    else:
+        libc = None
+    return libc
