@@ -2,9 +2,9 @@
 """Hooks for pygments syntax highlighting."""
 import os
 import re
+import sys
 import string
 import builtins
-from warnings import warn
 from collections import ChainMap
 from collections.abc import MutableMapping
 
@@ -24,14 +24,21 @@ from xonsh.commands_cache import CommandsCache
 from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
 from xonsh.tools import (ON_WINDOWS, intensify_colors_for_cmd_exe,
                          expand_gray_colors_for_cmd_exe)
+from xonsh.color_tools import (RE_BACKGROUND, BASE_XONSH_COLORS, make_pallete,
+                               find_closest_color)
+from xonsh.style_tools import norm_name
 from xonsh.lazyimps import terminal256
+from xonsh.platform import os_environ
 
 load_module_in_background('pkg_resources', debug='XONSH_DEBUG',
                           replacements={'pygments.plugin': 'pkg_resources'})
 
 
 def _command_is_valid(cmd):
-    cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
+    try:
+        cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
+    except (FileNotFoundError, OSError):
+        return False
     return cmd in builtins.__xonsh_commands_cache__ or \
         (os.path.isfile(cmd_abspath) and os.access(cmd_abspath, os.X_OK))
 
@@ -39,7 +46,10 @@ def _command_is_valid(cmd):
 def _command_is_autocd(cmd):
     if not builtins.__xonsh_env__.get('AUTO_CD', False):
         return False
-    cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
+    try:
+        cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
+    except (FileNotFoundError, OSError):
+        return False
     return os.path.isdir(cmd_abspath)
 
 
@@ -54,9 +64,11 @@ def subproc_cmd_callback(_, match):
 def subproc_arg_callback(_, match):
     """Check if match contains valid path"""
     text = match.group()
-    yield (match.start(),
-           Name.Constant if os.path.exists(os.path.expanduser(text)) else Text,
-           text)
+    try:
+        ispath = os.path.exists(os.path.expanduser(text))
+    except (FileNotFoundError, OSError):
+        ispath = False
+    yield (match.start(), Name.Constant if ispath else Text, text)
 
 
 COMMAND_TOKEN_RE = r'[^=\s\[\]{}()$"\'`<&|;!]+(?=\s|$|\)|\]|\}|!)'
@@ -75,7 +87,7 @@ class XonshLexer(PythonLexer):
         if not hasattr(builtins, '__xonsh_env__'):
             setattr(builtins, '__xonsh_env__', {})
             if ON_WINDOWS:
-                pathext = os.environ.get('PATHEXT', ['.EXE', '.BAT', '.CMD'])
+                pathext = os_environ.get('PATHEXT', ['.EXE', '.BAT', '.CMD'])
                 builtins.__xonsh_env__['PATHEXT'] = pathext.split(os.pathsep)
         if not hasattr(builtins, '__xonsh_commands_cache__'):
             setattr(builtins, '__xonsh_commands_cache__', CommandsCache())
@@ -198,14 +210,6 @@ class XonshConsoleLexer(XonshLexer):
 
 Color = Token.Color  # alias to new color token namespace
 
-RE_BACKGROUND = LazyObject(lambda: re.compile('(BG#|BGHEX|BACKGROUND)'),
-                           globals(), 'RE_BACKGROUND')
-
-
-def norm_name(name):
-    """Normalizes a color name."""
-    return name.replace('#', 'HEX').replace('BGHEX', 'BACKGROUND_HEX')
-
 
 def color_by_name(name, fg=None, bg=None):
     """Converts a color name to a color token, foreground name,
@@ -421,10 +425,13 @@ class XonshStyle(Style):
         if self._style_name == value:
             return
         if value not in STYLES:
-            warn('Could not find style {0!r}, using default'.format(value),
-                 RuntimeWarning)
-            value = 'default'
-            builtins.__xonsh_env__['XONSH_COLOR_STYLE'] = value
+            try:  # loading style dynamically
+                pygments_style_by_name(value)
+            except Exception:
+                print('Could not find style {0!r}, using default'.format(value),
+                      file=sys.stderr)
+                value = 'default'
+                builtins.__xonsh_env__['XONSH_COLOR_STYLE'] = value
         cmap = STYLES[value]
         if value == 'default':
             self._smap = XONSH_BASE_STYLE.copy()
@@ -729,7 +736,7 @@ def _default_style():
             Color.WHITE: '#ansilightgray',
             Color.YELLOW: '#ansibrown',
         }
-    elif ON_WINDOWS and 'CONEMUANSI' not in os.environ:
+    elif ON_WINDOWS and 'CONEMUANSI' not in os_environ:
         # These colors must match the color specification
         # in prompt_toolkit, so the colors are converted
         # correctly when using cmd.exe
@@ -1392,6 +1399,32 @@ del (_algol_style, _algol_nu_style, _autumn_style, _borland_style, _bw_style,
      _vim_style, _vs_style, _xcode_style)
 
 
+# dynamic syles
+def make_pygments_style(pallette):
+    """Makes a pygments style based on a color pallete."""
+    global Color
+    style = {getattr(Color, 'NO_COLOR'): 'noinherit'}
+    for name, t in BASE_XONSH_COLORS.items():
+        color = find_closest_color(t, pallette)
+        style[getattr(Color, name)] = '#' + color
+        style[getattr(Color, 'BOLD_'+name)] = 'bold #' + color
+        style[getattr(Color, 'UNDERLINE_'+name)] = 'underline #' + color
+        style[getattr(Color, 'BOLD_UNDERLINE_'+name)] = 'bold underline #' + color
+        style[getattr(Color, 'BACKGROUND_'+name)] = 'bg:#' + color
+    return style
+
+
+def pygments_style_by_name(name):
+    """Gets or makes a pygments color style by its name."""
+    if name in STYLES:
+        return STYLES[name]
+    pstyle = get_style_by_name(name)
+    pallette = make_pallete(pstyle.styles.values())
+    astyle = make_pygments_style(pallette)
+    STYLES[name] = astyle
+    return astyle
+
+
 #
 # Formatter
 #
