@@ -27,9 +27,10 @@ from xonsh.ansi_colors import (ansi_partial_color_format, ansi_color_style_names
                                ansi_color_style)
 from xonsh.prompt.base import multiline_prompt
 from xonsh.tools import (print_exception, check_for_partial_string, to_bool,
-                         columnize)
-from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_DARWIN, ON_POSIX
-from xonsh.lazyimps import pygments, pyghooks
+                         columnize, carriage_return)
+from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_DARWIN, ON_POSIX, os_environ
+from xonsh.lazyimps import pygments, pyghooks, winutils
+from xonsh.events import events
 
 readline = None
 RL_COMPLETION_SUPPRESS_APPEND = RL_LIB = RL_STATE = None
@@ -116,7 +117,7 @@ def setup_readline():
     else:
         readline.parse_and_bind("tab: complete")
     # try to load custom user settings
-    inputrc_name = os.environ.get('INPUTRC')
+    inputrc_name = os_environ.get('INPUTRC')
     if inputrc_name is None:
         if uses_libedit:
             inputrc_name = '.editrc'
@@ -125,12 +126,16 @@ def setup_readline():
         inputrc_name = os.path.join(os.path.expanduser('~'), inputrc_name)
     if (not ON_WINDOWS) and (not os.path.isfile(inputrc_name)):
         inputrc_name = '/etc/inputrc'
+    if ON_WINDOWS:
+        winutils.enable_virtual_terminal_processing()
     if os.path.isfile(inputrc_name):
         try:
             readline.read_init_file(inputrc_name)
         except Exception:
             # this seems to fail with libedit
             print_exception('xonsh: could not load readline default init file.')
+    # properly reset intput typed before the first prompt
+    readline.set_startup_hook(carriage_return)
 
 
 def teardown_readline():
@@ -229,6 +234,10 @@ def rl_on_new_line():
         func = getattr(RL_LIB, name, None)
         if func is not None:
             break
+    else:
+        def print_for_newline():
+            print()
+        func = print_for_newline
     return func
 
 
@@ -275,7 +284,9 @@ class ReadlineShell(BaseShell, cmd.Cmd):
             except ImportError:
                 store_in_history = True
             pos = readline.get_current_history_length() - 1
+        events.on_pre_prompt.fire()
         rtn = input(self.prompt)
+        events.on_post_prompt.fire()
         if not store_in_history and pos >= 0:
             readline.remove_history_item(pos)
         return rtn
@@ -326,19 +337,11 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         rl_completion_suppress_append()  # this needs to be called each time
         _rebind_case_sensitive_completions()
         rl_completion_query_items(val=999999999)
-        _s, _e, _q = check_for_partial_string(line)
-        if _s is not None:
-            if _e is not None and ' ' in line[_e:]:
-                mline = line.rpartition(' ')[2]
-            else:
-                mline = line[_s:]
-        else:
-            mline = line.rpartition(' ')[2]
-        offs = len(mline) - len(prefix)
-        completions = self.completer.complete(prefix, line,
-                                              begidx, endidx,
-                                              ctx=self.ctx)[0]
-        rtn_completions = [i[offs:] for i in completions]
+        completions, l = self.completer.complete(prefix, line,
+                                                 begidx, endidx,
+                                                 ctx=self.ctx)
+        chopped = prefix[:-l]
+        rtn_completions = [chopped + i for i in completions]
         show_completions = self._querycompletions(completions, endidx - begidx)
         return rtn_completions if show_completions else []
 
@@ -452,6 +455,8 @@ class ReadlineShell(BaseShell, cmd.Cmd):
                 line = self.precmd(line)
                 stop = self.onecmd(line)
                 stop = self.postcmd(stop, line)
+                if ON_WINDOWS:
+                    winutils.enable_virtual_terminal_processing()
             self.postloop()
         finally:
             if self.use_rawinput and self.completekey:
@@ -465,7 +470,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         while not builtins.__xonsh_exit__:
             try:
                 self._cmdloop(intro=intro)
-            except KeyboardInterrupt:
+            except (KeyboardInterrupt, SystemExit):
                 print()  # Gives a newline
                 fix_readline_state_after_ctrl_c()
                 self.reset_buffer()
@@ -572,7 +577,9 @@ class ReadlineHistoryAdder(threading.Thread):
         i = 1
         for h in hist.all_items():
             line = h['inp'].rstrip()
-            if line == readline.get_history_item(i - 1):
+            if i == 1:
+                pass
+            elif line == readline.get_history_item(i - 1):
                 continue
             readline.add_history(line)
             if RL_LIB is not None:
