@@ -204,6 +204,18 @@ class EnvPath(collections.MutableSequence):
             return False
         return all(map(operator.eq, self, other))
 
+    def _repr_pretty_(self, p, cycle):
+        """ Pretty print path list """
+        if cycle:
+            p.text('EnvPath(...)')
+        else:
+            with p.group(1, 'EnvPath(\n[', ']\n)'):
+                for idx, item in enumerate(self):
+                    if idx:
+                        p.text(',')
+                        p.breakable()
+                    p.pretty(item)
+
 
 class DefaultNotGivenType(object):
     """Singleton for representing when no default value is given."""
@@ -405,13 +417,13 @@ def _have_open_triple_quotes(s):
     return open_triple
 
 
-@lazyobject
-def LINE_CONTINUATION():
+def get_line_continuation():
     """ The line contiuation characters used in subproc mode. In interactive
-         mode on Windows the backslash must be preseeded by a space. This is because
-         paths on windows may end in a backspace.
+         mode on Windows the backslash must be preceded by a space. This is because
+         paths on Windows may end in a backslash.
     """
-    if ON_WINDOWS and builtins.__xonsh_env__.get('XONSH_INTERACTIVE'):
+    if (ON_WINDOWS and hasattr(builtins, '__xonsh_env__') and
+            builtins.__xonsh_env__.get('XONSH_INTERACTIVE', False)):
         return ' \\'
     else:
         return '\\'
@@ -425,7 +437,7 @@ def get_logical_line(lines, idx):
     """
     n = 1
     nlines = len(lines)
-    linecont = str(LINE_CONTINUATION)
+    linecont = get_line_continuation()
     while idx > 0 and lines[idx-1].endswith(linecont):
         idx -= 1
     start = idx
@@ -446,6 +458,7 @@ def replace_logical_line(lines, logical, idx, n):
     """Replaces lines at idx that may end in line continuation with a logical
     line that spans n lines.
     """
+    linecont = get_line_continuation()
     if n == 1:
         lines[idx] = logical
         return
@@ -459,7 +472,7 @@ def replace_logical_line(lines, logical, idx, n):
             logical = ''
         else:
             # found space to split on
-            lines[i] = logical[:b] + str(LINE_CONTINUATION)
+            lines[i] = logical[:b] + linecont
             logical = logical[b:]
     lines[idx+n-1] = logical
 
@@ -1719,12 +1732,6 @@ def POSIX_ENVVAR_REGEX():
     pat = r"""\$({(?P<quote>['"])|)(?P<envvar>\w+)((?P=quote)}|(?:\1\b))"""
     return re.compile(pat)
 
-if ON_WINDOWS:
-    # i.e %FOO%
-    @lazyobject
-    def WINDOWS_ENVVAR_REGEX():
-        return re.compile(r"%(?P<envvar>\w+)%")
-
 
 def expandvars(path):
     """Expand shell variables of the forms $var, ${var} and %var%.
@@ -1736,13 +1743,6 @@ def expandvars(path):
     elif isinstance(path, pathlib.Path):
         # get the path's string representation
         path = str(path)
-    if ON_WINDOWS and '%' in path:
-        for match in WINDOWS_ENVVAR_REGEX.finditer(path):
-            name = match.group('envvar')
-            if name in env:
-                ensurer = env.get_ensurer(name)
-                value = ensurer.detype(env[name])
-                path = WINDOWS_ENVVAR_REGEX.sub(value, path, count=1)
     if '$' in path:
         for match in POSIX_ENVVAR_REGEX.finditer(path):
             name = match.group('envvar')
