@@ -21,7 +21,6 @@ IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 """
 import os
-import sys
 import ctypes
 import subprocess
 from ctypes import c_ulong, c_char_p, c_int, c_void_p, POINTER, byref
@@ -142,10 +141,6 @@ def sudo(executable, args=None):
         nShow=SW_SHOW
     )
 
-    if not all(stream.isatty() for stream in (sys.stdin, sys.stdout, sys.stderr)):
-        # TODO: Some streams were redirected, we need to manually work them
-        raise NotImplementedError("Redirection is not supported")
-
     if not ShellExecuteEx(byref(execute_info)):
         raise ctypes.WinError()
 
@@ -249,6 +244,14 @@ def set_console_mode(mode, fd=1):
     SetConsoleMode(hcon, mode)
 
 
+def enable_virtual_terminal_processing():
+    """Enables virtual terminal processing on Windows.
+    This inlcudes ANSI escape sequence interpretation.
+    See http://stackoverflow.com/a/36760881/2312428
+    """
+    SetConsoleMode(GetStdHandle(-11), 7)
+
+
 @lazyobject
 def COORD():
     if platform.has_prompt_toolkit():
